#include "stdafx.h"
#include "titleinfo.h"

TitleInfo::TitleInfo() {
	strcpy(szText,"");
	setDefaultFont();
}

void TitleInfo::setText(const char *newtext) {
	strncpy(szText, newtext, 512); // incl. null character
}

void TitleInfo::setFont(const LOGFONT &newfont) {
	sCaptionFont = newfont;
}

void TitleInfo::setFontColor(const DWORD newcolor) {
	nCaptionFontColor = newcolor;
}

void TitleInfo::updateCaption(HWND hWnd) {
    RECT sWindowRect;
	HRGN hRgn;
    unsigned int nStyle;
	unsigned int nSizeY = 0;
	unsigned int nSizeX = 0;
    char szWindowTitle[256];

    GetWindowRect(hWnd, &sWindowRect);

    nStyle = GetWindowLong(hWnd, GWL_STYLE);

    // Y size
    if ((nStyle & WS_BORDER) == WS_BORDER && !((nStyle & WS_MAXIMIZE) == WS_MAXIMIZE)) {
        nSizeY += GetSystemMetrics(SM_CYBORDER);
    }
    if ((nStyle & WS_THICKFRAME) == WS_THICKFRAME && !((nStyle & WS_MAXIMIZE) == WS_MAXIMIZE)) {
        nSizeY += GetSystemMetrics(SM_CYFRAME);
    }
    if ((nStyle & WS_DLGFRAME) == WS_DLGFRAME && !((nStyle & WS_MAXIMIZE) == WS_MAXIMIZE)) {
        nSizeY += GetSystemMetrics(SM_CYDLGFRAME);
    }
	if ((nStyle & WS_CAPTION) == WS_CAPTION) {
		nSizeY += GetSystemMetrics(SM_CYCAPTION);
	}
	
    // X size
    nSizeX = 5;
    if ((nStyle & WS_SYSMENU) == WS_SYSMENU) {
        nSizeX += GetSystemMetrics(SM_CXSIZE);
    }
    if ((nStyle & WS_MINIMIZEBOX) == WS_MINIMIZEBOX || (nStyle & WS_MAXIMIZEBOX) == WS_MAXIMIZEBOX) {
        nSizeX += 2*GetSystemMetrics(SM_CXSIZE);
    }
    if ((nStyle & WS_BORDER) == WS_BORDER && !((nStyle & WS_MAXIMIZE) == WS_MAXIMIZE)) {
        nSizeX += GetSystemMetrics(SM_CXBORDER);
    }
    if ((nStyle & WS_THICKFRAME) == WS_THICKFRAME && !((nStyle & WS_MAXIMIZE) == WS_MAXIMIZE)) {
        nSizeX += GetSystemMetrics(SM_CXFRAME);
    }
    if ((nStyle & WS_DLGFRAME) == WS_DLGFRAME && !((nStyle & WS_MAXIMIZE) == WS_MAXIMIZE)) {
        nSizeX += GetSystemMetrics(SM_CXDLGFRAME);
    }
    
    sWindowRect.bottom = sWindowRect.top + nSizeY + 3;
    sWindowRect.right -= nSizeX;
    sWindowRect.left = nLastCaptionXPos - 20;

	GetWindowText(hWnd, szWindowTitle, 256);
/*
    DebugPrintf("UpdateCaption (%s): nSizeY = %d, nSizeX = %d, Region: %d, %d, %d, %d", 
                          szWindowTitle,
                          nSizeY, 
                          nSizeX,
                          sWindowRect.left, 
						  sWindowRect.top,
						  sWindowRect.right,
						  sWindowRect.bottom);
*/
	hRgn = CreateRectRgn(sWindowRect.left, 
						 sWindowRect.top,
						 sWindowRect.right,
						 sWindowRect.bottom);
//	SendMessage(hWnd, WM_NCPAINT, (WPARAM)hRgn, 0);
    DefWindowProc(hWnd, WM_NCPAINT, (WPARAM) hRgn, 0);

//    UpdateWindow(hWnd);

	DeleteObject(hRgn);
}

void TitleInfo::drawOnCaption() {

    HWND hForegroundWnd = GetForegroundWindow();
    char szName[80];
    unsigned int nStyle;
    unsigned int nExStyle;

    if (!hForegroundWnd || szText[0]=='\0') {
        return;
	}

    // Find out the actual parent of this foreground window 
    // in case it is a popup window with a caption
    if (GetParent(hForegroundWnd)) {
        HWND hWnd;

        hWnd = GetParent(hForegroundWnd);
        while(hWnd) {
            hForegroundWnd = hWnd;
            hWnd = GetParent(hForegroundWnd);
        }
    }

    // If we have changed the foreground window, 
    // repaint the last foreground window caption
    if (hForegroundWnd != hLastForegroundWindow && hLastForegroundWindow
			&& IsWindow(hLastForegroundWindow)) {
		updateCaption(hLastForegroundWindow);
    }

    // Check if we are supposed to draw this info
	// the window must have a titelbar and must not be a toolwindow
	// it must not be setup.exe or winver.exe (#32770) and not 
	// the Windows Taskbar (Shell_TrayWnd)
	// it also mustn't be a Winamp window (Winamp v1.x).
    GetClassName(hForegroundWnd, szName, 80);
    nStyle = GetWindowLong(hForegroundWnd, GWL_STYLE);
    nExStyle = GetWindowLong(hForegroundWnd, GWL_EXSTYLE);

    if ((nStyle & WS_CAPTION) == WS_CAPTION && 
			!(nExStyle & WS_EX_TOOLWINDOW) &&
			strcmp(szName, "#32770") && 
			strcmp(szName, "Shell_TrayWnd") &&
			strcmp(szName, "Winamp v1.x") &&
			strcmp(szName, "Winamp EQ") &&
			strcmp(szName, "Winamp PE") &&
			strcmp(szName, "Winamp MB")) {
		drawCaption(hForegroundWnd, nStyle);
    }
	hLastForegroundWindow = hForegroundWnd;
}

void TitleInfo::drawCaption(HWND hForegroundWnd, unsigned int nStyle) {
	char szTmp[512];
    HDC hDC;
    RECT sRect;
    SIZE sSize;
    SIZE sSize2;
    NONCLIENTMETRICS sMetrics;
    HFONT hDrawFont;
    signed int nLeft;
    unsigned int nLeftSpace = 0;
    unsigned int nRightSpace = 0;
    unsigned int nDrawCaptionPos = 0;
    RECT sDrawRect;
        
	hLastForegroundWindow = hForegroundWnd;
    hDC = GetWindowDC(hForegroundWnd);
    GetWindowText(hForegroundWnd, szTmp, 511);
    sMetrics.cbSize = sizeof(sMetrics);
    SystemParametersInfo(SPI_GETNONCLIENTMETRICS, 0, &sMetrics, FALSE);

    // First we need to calculate the actual length of the active window text
    hDrawFont = CreateFontIndirect(&sMetrics.lfCaptionFont);
    hDrawFont = (HFONT)SelectObject(hDC, hDrawFont);
    GetTextExtentPoint(hDC, szTmp, strlen(szTmp), &sSize2);
    hDrawFont = (HFONT)SelectObject(hDC, hDrawFont);
    DeleteObject(hDrawFont);

    // Now use the user defined font
    hDrawFont = CreateFontIndirect(&sCaptionFont);
    hDrawFont = (HFONT)SelectObject(hDC, hDrawFont);
    GetWindowRect(hForegroundWnd, &sRect);
    GetTextExtentPoint(hDC, szText, strlen(szText), &sSize);
    nRightSpace = 10;

//#define DEBUG_SHOW

#ifdef DEBUG_SHOW
    DebugPrintf("-----");
#endif
    // Close button
    if ((nStyle & WS_SYSMENU) == WS_SYSMENU) {
#ifdef DEBUG_SHOW
		DebugPrintf("WS_SYSMENU");
#endif
		nRightSpace += GetSystemMetrics(SM_CXSIZE);
		nLeftSpace += GetSystemMetrics(SM_CXSIZE);
	}
	if ((nStyle & WS_MINIMIZEBOX) == WS_MINIMIZEBOX
			|| (nStyle & WS_MAXIMIZEBOX) == WS_MAXIMIZEBOX) {
#ifdef DEBUG_SHOW
		DebugPrintf("WS_MINIMIZEBOX or WS_MAXIMIZEBOX");
#endif
		nRightSpace += 2*GetSystemMetrics(SM_CXSIZE);
	}
	if ((nStyle & WS_BORDER) == WS_BORDER
			&& !((nStyle & WS_MAXIMIZE) == WS_MAXIMIZE)) {
#ifdef DEBUG_SHOW
		DebugPrintf("WS_BORDER");
#endif
		nRightSpace += GetSystemMetrics(SM_CXBORDER);
		nLeftSpace += GetSystemMetrics(SM_CXBORDER);
		nDrawCaptionPos += GetSystemMetrics(SM_CXBORDER);
	}
	if ((nStyle & WS_THICKFRAME) == WS_THICKFRAME
			&& !((nStyle & WS_MAXIMIZE) == WS_MAXIMIZE)) {
#ifdef DEBUG_SHOW
		DebugPrintf("WS_THICKFRAME");
#endif
		nRightSpace += GetSystemMetrics(SM_CXFRAME);
		nLeftSpace += GetSystemMetrics(SM_CXFRAME);
		nDrawCaptionPos += GetSystemMetrics(SM_CXFRAME);
	}
	if ((nStyle & WS_DLGFRAME) == WS_DLGFRAME
			&& !((nStyle & WS_MAXIMIZE) == WS_MAXIMIZE)) {
#ifdef DEBUG_SHOW
		DebugPrintf("WS_DLGFRAME");
#endif
		nRightSpace += GetSystemMetrics(SM_CXDLGFRAME);
		nLeftSpace += GetSystemMetrics(SM_CXDLGFRAME);
		nDrawCaptionPos += GetSystemMetrics(SM_CXDLGFRAME);
	}
	nLeft = sRect.right - sRect.left - nRightSpace - sSize.cx;
	SetBkMode(hDC, TRANSPARENT);
	SetTextColor(hDC, nCaptionFontColor);
	sDrawRect.top = GetSystemMetrics(SM_CYFRAME) + (GetSystemMetrics(SM_CYCAPTION) - sSize.cy) / 2 - 1;
	sDrawRect.bottom = sDrawRect.top + sSize.cy;
	updateCaption(hForegroundWnd);
	nLastCaptionLen = sSize.cx;

	if (nLeft > (signed) (sSize2.cx + nLeftSpace + 30)) {
		sDrawRect.left = nLeft;
		sDrawRect.right = sDrawRect.left + sSize.cx;
#ifdef DEBUG_SHOW
		DebugPrintf("Text fits");
#endif
	} else {
		sDrawRect.left = sSize2.cx + 30 + nLeftSpace;
		sDrawRect.right = nLeft + sSize.cx;
#ifdef DEBUG_SHOW
		DebugPrintf("Text too large");
#endif
	}
	nLastCaptionXPos = sDrawRect.left;
	DrawText(hDC, szText, strlen(szText), &sDrawRect,
		DT_LEFT | DT_VCENTER | DT_NOPREFIX);
	hDrawFont = (HFONT)SelectObject(hDC, hDrawFont);
	DeleteObject(hDrawFont);
	ReleaseDC(hForegroundWnd, hDC);
}

void TitleInfo::setDefaultFont() {
	nCaptionFontColor = GetSysColor(COLOR_CAPTIONTEXT);//16777215;
	sCaptionFont.lfHeight=-11;
	sCaptionFont.lfWidth=0;
	sCaptionFont.lfEscapement=0;
	sCaptionFont.lfOrientation=0;
	sCaptionFont.lfWeight=400;
	sCaptionFont.lfItalic=0;
	sCaptionFont.lfUnderline=0;
	sCaptionFont.lfStrikeOut=0;
	sCaptionFont.lfCharSet=0;
	sCaptionFont.lfOutPrecision=1;
	sCaptionFont.lfClipPrecision=2;
	sCaptionFont.lfQuality=1;
	sCaptionFont.lfPitchAndFamily=34;
	strncpy(sCaptionFont.lfFaceName,"MS Sans Serif", LF_FACESIZE);
}